﻿using Hotline.Api.Filter;
using Hotline.Application.Orders;
using Hotline.Application.Snapshot;
using Hotline.File;
using Hotline.Orders;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Article;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Dtos.Snapshot;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Snapshot;
using Hotline.Share.Tools;
using Hotline.Snapshot;
using Hotline.Snapshot.Interfaces;
using Hotline.Tools;
using Mapster;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using SqlSugar;
using System.ComponentModel.DataAnnotations;
using System.Reflection;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers.Snapshot;

/// <summary>
/// 随手拍接口
/// </summary>
public class SnapshotController : BaseController
{
    private readonly IRepository<Order> _orderRepository;
    private readonly ISnapshotApplication _snapshotApplication;
    private readonly ISystemAreaDomainService _systemAreaDomainService;
    private readonly IIndustryRepository _industryRepository;
    private readonly IOrderDomainService _orderDomainService;
    private readonly IFileRepository _fileRepository;
    private readonly IOrderSnapshotRepository _orderSnapshotRepository;

    public SnapshotController(IRepository<Order> orderRepository, ISnapshotApplication snapshotApplication, ISystemAreaDomainService systemAreaDomainService, IIndustryRepository industryRepository, IOrderDomainService orderDomainService, IFileRepository fileRepository, IOrderSnapshotRepository orderSnapshotRepository)
    {
        _orderRepository = orderRepository;
        _snapshotApplication = snapshotApplication;
        _systemAreaDomainService = systemAreaDomainService;
        _industryRepository = industryRepository;
        _orderDomainService = orderDomainService;
        _fileRepository = fileRepository;
        _orderSnapshotRepository = orderSnapshotRepository;
    }

    /// <summary>
    /// 首页数据
    /// </summary>
    /// <returns></returns>
    [HttpGet("home")]
    [AllowAnonymous]
    public async Task<HomePageOutDto> GetHomePageAsync()
        => await _snapshotApplication.GetHomePageAsync();


    /// <summary>
    /// 行业界面基础信息
    /// </summary>
    /// <param name="id">行业Id</param>
    /// <returns></returns>
    [HttpGet("industry/base/{id}")]
    [AllowAnonymous]
    public async Task<IndustryBaseOutDto> GetIndustryBaseAsync(string id)
        => await _snapshotApplication.GetIndustryBaseAsync(id, HttpContext.RequestAborted);

    /// <summary>
    /// 添加随手拍工单
    /// </summary>
    /// <returns></returns>
    [HttpPost("order")]
    [LogFilter("添加随手拍工单")]
    public async Task<AddSnapshotOrderOutDto> AddOrderAsync([FromBody] AddSnapshotOrderInDto dto)
    {
        var order = dto.Adapt<Order>();
        dto.ValidateObject();
        var industry = await _industryRepository.GetAsync(dto.IndustryId, HttpContext.RequestAborted)
            ?? throw UserFriendlyException.SameMessage("行业不存在:" + dto.IndustryId);
        order.AcceptTypeCode = industry.AcceptTypeCode;
        order.AcceptType = industry.AcceptType;
        order.FromGender = EGender.Unknown;
        order.Title = dto.GetTitle(industry.IndustryType, industry.AcceptType);
        order.Content = dto.GetContent(industry.IndustryType);
        order.InitId();
        await _orderDomainService.AddAsync(order);
        if (dto.Files.NotNullOrEmpty())
            order.FileJson = await _fileRepository.AddFileAsync(dto.Files, order.Id, HttpContext.RequestAborted);
        var orderSnapshot = dto.Adapt<OrderSnapshot>();
        orderSnapshot.Id = order.Id;
        orderSnapshot.IndustryId = dto.IndustryId;
        orderSnapshot.IndustryName = industry.Name;
        await _orderSnapshotRepository.AddAsync(orderSnapshot);
        return order.Adapt<AddSnapshotOrderOutDto>();
    }

    /// <summary>
    /// 获取行业集合
    /// </summary>
    /// <returns></returns>
    [HttpGet("industry")]
    public async Task<IList<IndustryOutDto>> GetIndustresAsync()
        => await _snapshotApplication.GetIndustresAsync();

    /// <summary>
    /// 获取公开的工单集合
    /// </summary>
    /// <returns></returns>
    [HttpGet("order/published")]
    public async Task<IList<OrderPublishOutDto>> GetPublishOrderAsync([FromQuery] OrderPublishInDto dto)
        => await _snapshotApplication.GetOrderPublishAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 获取公开的工单详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpGet("order/published/{id}")]
    public async Task<OrderPublishDetailOutDto> GetOrderPublishDetailAsync(string id)
        => await _snapshotApplication.GetOrderPublishDetailAsync(id, HttpContext.RequestAborted);

    /// <summary>
    /// 获取小程序公告列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("bulletions")]
    [AllowAnonymous]
    public virtual async Task<IReadOnlyList<BulletinOutDto>> QueryBulletinsAsync([FromQuery] BulletinInDto dto)
        => await _snapshotApplication.GetBulletinsAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 公告详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpGet("bulletions/{id}")]
    [AllowAnonymous]
    public virtual async Task<BulletinOutDto> QueryBulletionsDetailAsync(string id)
        => await _snapshotApplication.GetBulletinsDetailAsync(id);

    /// <summary>
    /// 获取用户页面数据
    /// </summary>
    /// <returns></returns>
    [AllowAnonymous]
    [HttpGet("user")]
    public virtual async Task<SnapshotUserInfoOutDto> GetUserInfo()
        => await _snapshotApplication.GetSnapshotUserInfoAsync();

    /// <summary>
    /// 获取我提交的线索列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("order")]
    public virtual async Task<IList<OrderOutDto>> QueryOrderListAsync([FromQuery] OrderInDto dto)
        => await _snapshotApplication.GetSnapshotOrdersAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 获取我提交的线索详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpGet("order/{id}")]
    public virtual async Task<OrderPublishDetailOutDto> QueryOrderListAsync([FromQuery] string id)
        => await _snapshotApplication.GetSnapshotOrderDetailAsync(id, HttpContext.RequestAborted);

    /// <summary>
    /// 统计红包金额, 每月的总金额
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("redpack")]
    public virtual async Task<IReadOnlyList<RedPackDateOutDto>> QueryRedPackDateAsync([FromQuery] RedPackDateInDto dto)
        => await _snapshotApplication.GetRedPackDateAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 用户领取红包总金额
    /// </summary>
    /// <returns></returns>
    [HttpGet("redpack/received")]
    public async Task<string> GetRedPackReceivedTotalAsync()
        => await _snapshotApplication.GetRedPackReceivedTotalAsync(HttpContext.RequestAborted);

    /// <summary>
    /// 获取当月详细红包列表
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("redpack/month")]
    public virtual async Task<IList<RedPackOutDto>> QueryRedPackDateAsync([FromQuery] RedPacksInDto dto)
        => await _snapshotApplication.GetRedPacksAsync(dto, HttpContext.RequestAborted);

    /// <summary>
    /// 获取随手拍电气焊动火作业待处理工单数量
    /// TODO 条件 电气焊作业申报
    /// </summary>
    /// <returns></returns>
    [HttpGet("wait_accept_count")]
    public async Task<int> GetSnapshotWaitForAcceptCountAsync()
        => await _orderRepository
            .CountAsync(m => m.SourceChannelCode == "ZGSSP" && m.Status == Share.Enums.Order.EOrderStatus.WaitForAccept);

    /// <summary>
    /// 获取区域
    /// </summary>
    /// <returns></returns>
    [HttpGet("area/tree")]
    public async Task<List<SystemAreaOutDto>> GetAreaTreeAsync()
        => (await _systemAreaDomainService.GetAreaTree()).Adapt<List<SystemAreaOutDto>>();

    /// <summary>
    /// 批量添加从业人员
    /// </summary>
    /// <param name="dtos"></param>
    /// <returns></returns>
    [HttpPost("practitioner")]
    public async Task AddPractitionerAsync([FromBody]IList<AddPractitionerInDto> dtos)
        => await _snapshotApplication.AddPractitionerAsync(dtos);

    /// <summary>
    /// 获取从业人员详情
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpGet("practitioner/{id}")]
    public async Task<PractitionerDetailOutDto> GetPractitionerDetailAsync(string id)
        => await _snapshotApplication.GetPractitionerDetailAsync(id, HttpContext.RequestAborted);

    /// <summary>
    /// 获取从业人员集合(每次随机)
    /// </summary>
    /// <param name="dto"></param>
    /// <returns></returns>
    [HttpGet("practitioner")]
    public async Task<IList<PractitionerItemOutDto>> GetPractitionerDetailAsync(PractitionerItemInDto dto)
        => await _snapshotApplication.GetPractitionerItemsAsync(dto, CancellationToken.None);
}
